/// @file   Queries.cpp
/// @brief  Oracle queries.
/// @author Alejandro Álvarez Ayllón <aalvarez@cern.ch>
#include "Queries.h"

const char* STMT_GET_FILE_BY_ID =
  "SELECT fileid, parent_fileid, guid, name, filemode, nlink, owner_uid, gid,\
          filesize, atime, mtime, ctime, fileclass, status,\
          csumtype, csumvalue, acl\
        FROM Cns_file_metadata\
        WHERE fileid = :b_fileid";
const char* STMT_GET_FILE_BY_GUID =
  "SELECT fileid, parent_fileid, guid, name, filemode, nlink, owner_uid, gid,\
          filesize, atime, mtime, ctime, fileclass, status,\
          csumtype, csumvalue, acl\
        FROM Cns_file_metadata\
        WHERE guid = :b_guid";
const char* STMT_GET_FILE_BY_NAME =
  "SELECT fileid, parent_fileid, guid, name, filemode, nlink, owner_uid, gid,\
          filesize, atime, mtime, ctime, fileclass, status,\
          csumtype, csumvalue, acl\
        FROM Cns_file_metadata \
        WHERE parent_fileid = :b_parent AND name = :b_name";
const char* STMT_GET_LIST_FILES =
  "SELECT fileid, parent_fileid, guid, name, filemode, nlink, owner_uid, gid,\
          filesize, atime, mtime, ctime, fileclass, status,\
          csumtype, csumvalue, acl\
        FROM Cns_file_metadata \
        WHERE parent_fileid = :b_parent";
const char* STMT_GET_SYMLINK =
  "SELECT fileid, linkname FROM Cns_symlinks WHERE fileid = :b_fileid";
const char* STMT_GET_FILE_REPLICAS =
  "SELECT DBMS_ROWID.ROWID_ROW_NUMBER(rowid), fileid, nbaccesses,\
          atime, ptime, ltime, status, f_type, poolname, host, fs, sfn\
        FROM Cns_file_replica\
        WHERE fileid = :b_fileid";
const char* STMT_GET_REPLICA_BY_URL =
  "SELECT DBMS_ROWID.ROWID_ROW_NUMBER(rowid), fileid, nbaccesses,\
          atime, ptime, ltime, status, f_type, poolname, host, fs, sfn\
        FROM Cns_file_replica\
        WHERE sfn = :b_sfn";
const char* STMT_GET_REPLICA_BY_ID =
  "SELECT DBMS_ROWID.ROWID_ROW_NUMBER(rowid), fileid, nbaccesses,\
          atime, ptime, ltime, status, f_type, poolname, host, fs, sfn\
        FROM Cns_file_replica\
        WHERE DBMS_ROWID.ROWID_ROW_NUMBER(rowid) = :b_id";
const char* STMT_GET_COMMENT =
  "SELECT comments\
        FROM Cns_user_metadata\
        WHERE u_fileid = :b_fileid";
const char* STMT_SET_GUID =
  "UPDATE Cns_file_metadata\
        SET guid = :b_guid\
        WHERE fileid = :b_fileid";
const char* STMT_SET_COMMENT =
  "UPDATE Cns_user_metadata\
        SET comments = :b_comment\
        WHERE u_fileid = :b_fileid";
const char* STMT_INSERT_COMMENT =
  "INSERT INTO Cns_user_metadata\
            (u_fileid, comments)\
        VALUES\
            (:b_fileid, :b_comment)";
const char* STMT_INSERT_FILE =
  "INSERT INTO Cns_file_metadata\
            (fileid, parent_fileid, name, filemode, nlink, owner_uid, gid,\
            filesize, atime, mtime, ctime, fileclass, status,\
            csumtype, csumvalue, acl)\
        VALUES\
            (Cns_unique_id.nextval, :b_parentid, :b_name, :b_mode, :b_nlink, :b_uid, :b_gid,\
            :b_size, :b_atime, :b_mtime, :b_ctime, :b_fclass, :b_status,\
            :b_sumtype, :b_sumvalue, :b_acl)";
const char* STMT_INSERT_SYMLINK =
  "INSERT INTO Cns_symlinks\
            (fileid, linkname)\
        VALUES\
            (:b_fileid, :b_linkname)";
const char* STMT_DELETE_FILE =
  "DELETE FROM Cns_file_metadata WHERE fileid = :b_fileid";
const char* STMT_DELETE_COMMENT =
  "DELETE FROM Cns_user_metadata WHERE u_fileid = :b_fileid";
const char* STMT_DELETE_SYMLINK =
  "DELETE FROM Cns_symlinks WHERE fileid = :b_fileid";
const char* STMT_SELECT_NLINK_FOR_UPDATE =
  "SELECT nlink FROM Cns_file_metadata WHERE fileid = :b_fileid FOR UPDATE";
const char* STMT_UPDATE_NLINK =
  "UPDATE Cns_file_metadata\
        SET nlink = :b_nlink, mtime = :b_mtime, ctime = :b_ctime\
        WHERE fileid = :b_fileid";
const char* STMT_UPDATE_PERMS =
  "UPDATE Cns_file_metadata\
        SET owner_uid = :b_uid, gid = :b_gid, filemode = :b_mode, acl = :b_acl, ctime = :b_ctime\
        WHERE fileid = :b_fileid";
const char* STMT_DELETE_REPLICA =
  "DELETE FROM Cns_file_replica\
        WHERE fileid = :b_fileid AND sfn = :b_sfn";
const char* STMT_ADD_REPLICA =
  "INSERT INTO Cns_file_replica\
            (fileid, nbaccesses,\
            ctime, atime, ptime, ltime,\
            r_type, status, f_type,\
            setname, poolname, host, fs, sfn)\
        VALUES\
            (:b_fileid, 0,\
            :b_ctime, :b_atime, :b_ptime, :b_ltime,\
            :b_rtype, :b_status, :b_ftype,\
            :b_setname, :b_pool, :b_host, :b_fs, :b_sfn)";
const char* STMT_TRUNCATE_FILE =
  "UPDATE Cns_file_metadata\
        SET filesize = 0, ctime = :b_ctime\
        WHERE fileid = :b_fileid";
const char* STMT_CHANGE_NAME =
  "UPDATE Cns_file_metadata\
        SET name = :b_name, ctime = :b_ctime\
        WHERE fileid = :b_fileid";
const char* STMT_CHANGE_PARENT =
  "UPDATE Cns_file_metadata\
        SET parent_fileid = :b_parent, ctime = :b_ctime\
        WHERE fileid = :b_fileid";
const char* STMT_UTIME =
  "UPDATE Cns_file_metadata\
        SET atime = :b_atime, mtime = :b_mtime, ctime = :b_ctime\
        WHERE fileid = :b_fileid";
const char* STMT_UPDATE_REPLICA =
  "UPDATE Cns_file_replica\
        SET atime = :b_atime, ltime = :b_ltime, nbaccesses = :b_nacc,\
            status = :b_status, f_type = :b_type\
        WHERE sfn = :b_sfn";

// Authn

const char* STMT_GET_ALL_USERS =
    "SELECT userid, username, user_ca, NVL(banned, 0)\
        FROM Cns_userinfo";
const char* STMT_GET_USERINFO_BY_NAME =
  "SELECT userid, username, user_ca, NVL(banned, 0)\
        FROM Cns_userinfo\
        WHERE username = :b_username";
const char* STMT_GET_USERINFO_BY_UID =
  "SELECT userid, username, user_ca, NVL(banned, 0)\
        FROM Cns_userinfo\
        WHERE userid = :b_uid";
const char* STMT_GET_ALL_GROUPS =
    "SELECT gid, groupname, NVL(banned, 0)\
        FROM Cns_groupinfo";
const char* STMT_GET_GROUPINFO_BY_NAME =
  "SELECT gid, groupname, NVL(banned, 0)\
        FROM Cns_groupinfo\
        WHERE groupname = :b_groupname";
const char* STMT_GET_GROUPINFO_BY_GID =
  "SELECT gid, groupname, NVL(banned, 0)\
        FROM Cns_groupinfo\
        WHERE gid = :b_gid";
const char* STMT_GET_UNIQ_UID_FOR_UPDATE =
  "SELECT id FROM Cns_unique_uid FOR UPDATE";
const char* STMT_GET_UNIQ_GID_FOR_UPDATE =
  "SELECT id FROM Cns_unique_gid FOR UPDATE";
const char* STMT_UPDATE_UNIQ_UID =
  "UPDATE Cns_unique_uid SET id = :b_uid";
const char* STMT_UPDATE_UNIQ_GID =
  "UPDATE Cns_unique_gid SET id = :b_gid";
const char* STMT_INSERT_UNIQ_UID =
  "INSERT INTO Cns_unique_uid (id) VALUES (:b_uid)";
const char* STMT_INSERT_UNIQ_GID =
  "INSERT INTO Cns_unique_gid (id) VALUES (:b_gid)";
const char* STMT_INSERT_USER =
  "INSERT INTO Cns_userinfo\
            (userid, username, user_ca, banned)\
        VALUES\
            (:b_uid, :b_username, :b_ca, :b_banned)";
const char* STMT_INSERT_GROUP =
  "INSERT INTO Cns_groupinfo\
            (gid, groupname, banned)\
        VALUES\
            (:b_gid, :b_groupname, :b_banned)";
const char* STMT_UPDATE_USER = 
  "UPDATE Cns_userinfo\
      SET banned = :b_banned\
      WHERE username = :b_username";
const char* STMT_UPDATE_GROUP = 
  "UPDATE Cns_groupinfo\
      SET banned = :b_banned\
      WHERE groupname = :b_groupname";
const char* STMT_DELETE_USER =
  "DELETE FROM Cns_userinfo\
      WHERE username = :b_username";
const char* STMT_DELETE_GROUP =
  "DELETE FROM Cns_groupinfo\
      WHERE groupname = :b_groupname";
